#
# adesso-fishing.R, 18 Dec 22
#
# Data from:
# An empirical study on a single company’s cost estimations of 338 software projects
# Christian Schürhoff, Stefan Hanenberg and Volker Gruhn1
#
# Analysed for a post on https://shape-of-code.com

library("colorspace")
library("plyr")


par(bty="l")
par(las=1)
par(pch="+")
pal_col=rainbow(4)


adesso=read.csv("adesso.csv", as.is=TRUE)

# ProjektNo values cluster in four bands
adesso$col_str=ifelse(adesso$ProjektNo < 3000, pal_col[1],
		ifelse(adesso$ProjektNo < 5500, pal_col[2],
		ifelse(adesso$ProjektNo < 7000, pal_col[3], pal_col[4])))


# Number of estimates for each project
num_est=count(adesso$ProjektNo)
plot(num_est)

# Number of projects estimated a given number of times
plot(count(count(adesso$ProjektNo)$freq))

plot(adesso$Perc_Runtime, adesso$CPI, ylim=c(0, 4))


# For each project, CPI during implementation
plot(1, type="n",
	xlim=c(0, 1), ylim=c(0.2, 2.5),
	xlab="Runtime (fraction)", ylab="CPI")

d_ply(adesso, .(ProjektNo), function(df) lines(df$Perc_Runtime, df$CPI))

# Number of estimates against days for each Project
plot(1, type="n",
	xaxs="i", yaxs="i",
	xlim=c(0, max(adesso$Num_Days)), ylim=c(0, 85),
	xlab="Duration (days)", ylab="CPI estimates")

d_ply(adesso, .(ProjektNo), function(df) points(df$Num_Days[1], nrow(df), col=df$col_str[1]))

a8109=subset(adesso, ProjektNo == 8109)
plot(a8109$Perc_Runtime, a8109$CPI)

length(which(num_est$freq > 31))
top_project=subset(num_est, freq > 44)

par(mfcol=c(4, 4))
par(oma=c(2, 3, 0.5, 0.5))
par(mar=c(3.9, 4.1, 0.4, 0.5))

many_est=subset(adesso, ProjektNo %in% top_project$x)

d_ply(many_est, .(ProjektNo), function(df) plot(df$Perc_Runtime, df$CPI, cex=1.4, col="red",
						cex.axis=1.4, cex.lab=1.3,
						xaxs="i",
						xlab="Project duration (%)", ylab="CPI"))


#
p_once=subset(adesso, !duplicated(ProjektNo))
sum(p_once$Num_Days)
# [1] 132393
sum(p_once$Num_Days)/250
# [1] 529.572
sum(p_once$Num_Days)/nrow(p_once)
# [1] 391.6953
sum(p_once$Num_Days)/nrow(p_once)/250
# [1] 1.566781
mean(p_once$Final_CPI)
# [1] 1.017021
sd(p_once$Final_CPI)
# [1] 0.302432

# Weighted mean
sum(adesso$CPI*adesso$Num_Days)/sum(adesso$Num_Days)
# [1] 0.9573186


xm=weighted.mean(adesso$CPI, adesso$Num_Days)
# [1] 0.9573186
wt=adesso$Num_Days/sum(adesso$Num_Days)
# Weighted standard deviation
v=sum(wt * (adesso$CPI - xm)^2)
sqrt(v)


